#include "config.h"
#include "utils.h"
#include <ctype.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <stdlib.h>

#ifndef HAVE_ERR
#include <stdarg.h>
void err(int eval,const char *fmt,...)
{
  va_list ap;
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  fprintf(stderr,": %s\n",strerror(errno));
  exit(eval);
}
#endif

#ifndef HAVE_ERRX
#include <stdarg.h>
void errx(int eval,const char *fmt,...)
{
  va_list ap;
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  fprintf(stderr,"%s\n",strerror(errno));
  va_end(ap);
  exit(eval);
}
#endif

#ifndef HAVE_WARN
#include <stdarg.h>
void	warn(const char *fmt, ...)
{
    va_list args;
    va_start(args,fmt);
    vfprintf(stderr,fmt, args);
    fprintf(stderr,": %s\n",strerror(errno));
}
#endif

#ifndef HAVE_WARNX
#include <stdarg.h>
void warnx(const char *fmt,...)
{
  va_list ap;
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
}
#endif

#ifndef HAVE_ISHEXNUMBER
int ishexnumber(int c)
{
    switch(c){
    case '0':         case '1':         case '2':         case '3':         case '4':
    case '5':         case '6':         case '7':         case '8':         case '9':
    case 'A':         case 'B':         case 'C':         case 'D':         case 'E':
    case 'F':         case 'a':         case 'b':         case 'c':         case 'd':
    case 'e':         case 'f':
	return 1;
    }
    return 0;
}
#endif

/**
 * Prints some of a buffer, but only printable characters...
 */
void fputascii(FILE *f,unsigned char *buf,int len)
{
    for(;len>0;len--,buf++){
	if(*buf == '|' || !isprint(*buf)){
	    fputc('.',f);
	}
	else {
	    fputc(*buf,f);
	}
    }
}

/* Figure out what to do for lseek */
#if defined(HAVE_LSEEKO)
#define LSEEK lseeko
#endif

#if defined(HAVE_LSEEKO64) && !defined(LSEEK)
#define LSEEK lseeko64
#endif

#if defined(HAVE_LSEEK64) && !defined(LSEEK)
#define LSEEK lseek64
#endif

#if !defined(LSEEK)
#define LSEEK lseek
#endif

#undef DEBUG_FILESIZE_TEST
//#define DEBUG_FILESIZE_TEST
int64_t get_filesize(int fd)
{
    assert(sizeof(int64_t)==8);

    /* Use ftello or ftello64 if we have them and if they return >0. Otherwise just probe. */
    /* TK */

    char buf[64];
    u_int64_t raw_filesize = 0;
    /* Phase 1; figure out how far we can seek... */
    int bits = 0;
    int i =0;
    for(bits=0;bits<60;bits++){
	raw_filesize = ((u_int64_t)1<<bits);
	if((LSEEK(fd,raw_filesize,SEEK_SET)<0) || (read(fd,buf,1)!=1)){
#ifdef DEBUG_FILESIZE_TEST
	    printf("fail: bits=%d raw_filesize=%"PRIx64"\n",bits,raw_filesize);
#endif
	    break;
	}
#ifdef DEBUG_FILESIZE_TEST
	printf("ok: bits=%d raw_filesize=%"PRIx64"\n",bits,raw_filesize);
#endif
    }
    if(bits==60) errx(1,"Partition detection not functional.\n");

    /* Phase 2; blank bits as necessary */
    for(i=bits;i>=0;i--){
	u_int64_t test = (u_int64_t)1<<i;
	u_int64_t test_filesize = raw_filesize | ((u_int64_t)1<<i);
#ifdef DEBUG_FILESIZE_TEST
	printf("\ni=%d raw_filesize=%"PRIx64" (%"PRIu64") \n   test_filesize=%"PRIx64"  test=%"PRIx64"\n",
	       i,(int64_t)raw_filesize,(int64_t)raw_filesize,(int64_t)test_filesize,(int64_t)test);
#endif
	if((LSEEK(fd,test_filesize,SEEK_SET)>=0) &&
	   (read(fd,buf,1)==1)){
#ifdef DEBUG_FILESIZE_TEST
	    printf("test passed\n");
#endif
	    raw_filesize |= test;
	} else{
	    raw_filesize &= ~test;
	}
#ifdef DEBUG_FILESIZE_TEST
	printf("i=%d raw_filesize=%"PRIu64"\n",i,raw_filesize);
#endif
    }
    if(raw_filesize>0) raw_filesize+=1;	/* seems to be needed */
    return raw_filesize;
}

